//
// File:        NetProcess.java
// Author:      Tomasz Stachowiak
// e-mail:      <stach@npac.syr.edu>
// Created:     Tue Jan 14 1997
// Time-stamp: <97/01/29 18:59:40 stach>
// Copyright Tomasz Stachowiak 1997 NPAC, Syracuse University

// Description:

import java.util.*;
import java.net.*;
import java.io.*;
import java.awt.*;

public class NetProcess implements Runnable{
  static int ADD = 1;
  static int DELETE = 2;
  static int port = 23694;

  Hashtable members;

  DatagramSocket socket;

  List memberList;

  public NetProcess(List list) {
    members = new Hashtable();
    try {
      socket = new DatagramSocket(port);
    } catch (SocketException e) {
      System.err.println("NetProcess: " + e);
    }
    memberList = list;
  }

  protected void finalize() {
    socket.close();
  }

  private void processMessage(byte[] msg) {
    byte type = msg[0];

    switch(type) {
    case 1:        // ADD
      add(msg);
      break;

    case 2:        // DELETE
      delete(msg);
      break;

    default:
      System.err.println("NetProcess: wrong message type");
    }
  }

  public void add(String name, String host) {
    try {
      Member mbr = new Member(name, host);
      members.put(host, mbr);
    } catch (UnknownHostException e) {
      System.err.println("NetProcess: " + e);
    }
  }

  private void add(byte[] msg) {
    String name = new String(msg, 0, 1, 20);
    String host = new String(msg, 0, 21, 50);
    memberList.addItem(host);
    try {
      Member mbr = new Member(name, host);
      members.put(host, mbr);
    } catch (UnknownHostException e) {
      System.err.println("NetProcess: " + e);
    }
  }

  private void delete(byte[] msg) {
    String host = new String(msg, 0, 21, 50);
    members.remove(host);

    int i = memberList.countItems() - 1;
    while(i >= 0) {
      if(memberList.getItem(i).compareTo(host) == 0)
	memberList.delItem(i);
      i--;
    }
  }

  public void SendToAll(byte[] msg) {
    Enumeration e = members.elements();

    for(; e.hasMoreElements(); ) {
      Member mbr = (Member)e.nextElement();
      send(msg, mbr.getAddr());
    }
  }

  private void send(byte[] msg, InetAddress addr) {
    DatagramPacket packet = new DatagramPacket(msg, msg.length, addr, 23657);
    try {
      socket.send(packet);
    } catch (IOException e) {
      System.err.println("NetProcess: " + e);
    }
  }

  public void run() {
    byte[] buffer = new byte[100];
    DatagramPacket packet = new DatagramPacket(buffer, buffer.length);

    while(true) {
      try {
	socket.receive(packet);
	processMessage(buffer);
      } catch (IOException e) {
	System.err.println("NetProcess: " + e);
      }
    }      
  }
}






